# AdminAgent.pm DataBase Administration Utility

use AnyDBM_File; 
use POSIX(O_CREAT,O_RDWR,O_RDONLY); 

# Return Values: 0 = OK
#                1 = Value does just exist, no error condition
# Where should the database live in ?
$DBPATH="/opt/sysadmin/databases/";
# Should the subject field be CaseSensitive or Not
#        Case          = CaseSensitive 
#        SomeThingElse = CaseSensitive
#        ToUpper       = Not Case Sensitive automatic conversion to upper case
#        ToLower       = Not Case Sensitive automatic conversion to lower case
$CaseInSubject = "ToUpper" ;
# Max Number of Actions allowed
$MaxActions = 1000 ;
# Debugging, everything else than 0 is On
$Debug = 0 ; # 

$MsgIdList    = $DBPATH."MsgIdList" ;
$UserAuth     = $DBPATH."UserAuth" ;
$ProgramReg   = $DBPATH."ProgramReg" ;
$Actions      = $DBPATH."Actions" ;

my $Actions_Iterator ;

my %MIL ;
my %UA ;
my %SCR ;
my %ACT ;

#-------------------- open/close the database -----------------------
sub open_database {
    my ( $writable ) = @_ ;
    my ( $Permissions ) = $writable ? O_CREAT|O_RDWR : O_RDONLY;

    tie(%MIL,AnyDBM_File,$MsgIdList ,$Permissions,0644) || die "tie: $!\n" ;
    tie(%UA ,AnyDBM_File,$UserAuth  ,$Permissions,0644) || die "tie: $!\n" ;
    tie(%SCR ,AnyDBM_File,$ProgramReg,$Permissions,0644) || die "tie: $!\n" ;
    tie(%ACT,AnyDBM_File,$Actions   ,$Permissions,0644) || die "tie: $!\n" ;

    1;
}

sub close_database {
    untie(%MIL) ;
    untie(%UA) ;
    untie(%SCR) ;
    untie(%ACT) ;
}

#-------------------- dump the database  -----------------------
sub dump_database {
	&dump_MsIDList();
	&dump_Auth();
	&dump_Scripts();
	&dump_Actions();
}
sub dump_MsIDList {
    print "Dump of MIL:\n" ;
    foreach ( keys %MIL) {
       print "$_     \t $MIL{$_} \n" ;
    }
}
sub dump_Auth {
    print "Dump of UA:\n" ;
    foreach ( keys %UA) {
       print "$_     \t $UA{$_} \n" ;
    }
}
sub dump_Scripts {
    print "Dump of SCR:\n" ;
    foreach ( keys %SCR) {
       print "$_     \t $SCR{$_} \n" ;
    }
}
sub dump_Actions {
    print "Dump of ACT:\n" ;
    foreach ( keys %ACT) {
       print "$_     \t $ACT{$_} \n" ;
    }
}
#-------------------- reset the database -----------------------
sub reset_MIL {
    print "Resetting MIL . . .\n" ;
    foreach ( keys %MIL) {
       delete $MIL{$_};
    }
}
sub reset_UA {
    print "Resetting UA . . .\n" ;
    foreach ( keys %UA) {
       delete $UA{$_};
    }
}
sub reset_ACT {
    print "Resetting ACT . . .\n" ;
    foreach ( keys %ACT) {
       delete $ACT{$_};
    }
}
sub reset_SCR {
    print "Resetting SCR . . .\n" ;
    foreach ( keys %SCR) {
       delete $SCR{$_};
    }
}
sub reset_database {
    &reset_MIL();
	&reset_UA();
    &reset_SCR();
    &reset_ACT();
}

#-------------------- add an entry -----------------------
sub add_MsgId {
    my ( $MsgId,$Subject,$Action,$Param,$Date,$Script ) = @_ ;

    $Subject = Case($Subject) ;
    my $Description = join(":",$Subject,$Action,$Param,$Date,$Script) ;
    print "$MsgId ======> $Description \n" ;
    
    print "ID : $MsgId \n" ;
    $MIL{$MsgId} = $Description ;
    return 0 ;
}

sub add_Auth {
    my ( $UserId,$ScriptId) = @_ ;

    if ( $UA{$UserId} ) {
       @ScriptIds = split(':',$UA{$UserId});
       foreach $id (@ScriptIds) {
          return 2 if ( $id eq $ScriptId ) ;
       }
       push(@ScriptIds,$ScriptId);
       $UA{$UserId} = join(':',@ScriptIds) ;
    } else {
       $UA{$UserId} = $ScriptId ;
    }
    return 1 ;
}

sub add_Script {
    my ( $ScriptID,$Script) = @_ ;

    if ( defined($SCR{$ScriptID}) ) {
	   return($ScriptID) ;
	} else {
       $SCR{$ScriptID} = $Script ;
	   return (0);
	}
}

sub add_Action {
    my ( $Subject,$Command,$Param,$ActionId ) = @_ ;

    $Subject = Case($Subject) ;
    $Action = $Command.";".$Param ;
    if ( $ACT{$Subject} ) {
       @Actions=split(':',$ACT{$Subject});
       foreach $Line (@Actions) {
		   my $T = $Line ;
		   $T =~ s/;([^;]+$)// ;  # Strip away the ActionId
		   my $ActId = $1 ;
           return ($ActId) if ( $T eq $Action) ; # it exists !!!
       }
	   $Action = $Action.";".$ActionId ;
       push(@Actions,$Action);
       $ACT{$Subject} = join(':',@Actions) ;
    } else {
	   $Action = $Action.";".$ActionId ;
       $ACT{$Subject} = $Action ;
    }
	return (0) ; # OK.
    
}

#-------------------- delete an entry -----------------------

sub del_Action {
    my ( $Subject,$Command,$Param) = @_ ;
    $Subject = Case($Subject) ;

	print "Deleting: $Subject,$Command,$Param \n" ;
    my $i ;

    $Action = $Command.";".$Param ;

    @Actions=split(':',$ACT{$Subject});
    for ( $i=0 ; $i < $#Actions + 1 ; $i++ ) {
        $Row = $Actions[$i] ;
        $Row =~ s/;([^;]+$)// ;
        $ActionId = $1 ;
        if ( $Action eq $Row ) {
          $Ele = splice(@Actions,$i,1);
          print "Element $Ele spliced \n" ;
          last ;
        }
    }
	print "Remaining Rows: $#Actions \n" ;
    if ( $#Actions + 1 > 0 ) {
       $ACT{$Subject} = join(':',@Actions) ;
    } else {
       delete $ACT{$Subject};
    }
	return ( $ActionId ) ;
}

sub del_Pgrm {
    my ( $PgrmId ) = @_ ;

	delete $SCR{$PgrmId} ;
}


sub del_MsgId {
    my ( $MsgId ) = @_ ;

    if ( $MIL{$MsgId} ) {
       delete $MIL{$MsgId} ;
       return 1 ;
    } else {
       return 0 ;
    }
}

sub del_User {
    my ( $UsrId ) = @_ ;

    if ( $UA{$UsrId} ) {
       delete $UA{$UsrId} ;
       return 1 ;
    } else {
       return 0 ;
    }
}

sub del_Auth {
    my ( $UserId,$ScriptId) = @_ ;
	my $i ;
	my $OldId ;
	my @ScriptIds ;

    print "Deleting: $UserId $ScriptId \n" if ($Debug) ;
    if ( $UA{$UserId} ) {
       @ScriptIds = split(':',$UA{$UserId});
	   print "Limit: $#ScriptIds \n" ;
	   for ( $i=0 ;  $i < $#ScriptIds +1 ; $i++ ) {
		  if ( $ScriptId == $ScriptIds[$i] ) {
              $OldId  = splice(@ScriptIds,$i,1);
			  print "$OldId removed\n" if ($Debug) ;
			  last ;
		  }
       }

	   if ( defined($OldId) ) {
	      print "NEW LENGTH NOW = $#ScriptIds  \n" ;
		  
		  if ( $#ScriptIds < 0  ) {
		     print "USER:  $UA{$UserId} \n" if ($Debug) ;
			 delete  $UA{$UserId} ;
		  } else {
			 $UA{$UserId} = join(":",@ScriptIds);
          }

		  return ( $OldId ) ;
		} else {
		  return ( -2 ) ;
		}


    } else {
       return ( -1 ) ;
    }
}

#-------------------- Does it exist ? -----------------------

sub is_auth {
    my ( $UserId,$ScriptId ) = @_ ;

    if ( $UA{$UserId} ) {
       @ScriptIds = split(':',$UA{$UserId}) ;
       foreach $Id (@ScriptIds) {
         if ( $Id eq $ScriptId ) {
            return 1 ;
         }
       }
    }
}

sub exist_Subject {
    my ( $Subject ) = @_ ;
    $Subject = Case($Subject) ;

    return 1 if $ACT{$Subject} ;
    return 0 ;
}

sub exist_MsgId {
    my ( $MsgId ) = @_ ;
    return  1 if ( $MIL{$MsgId} );
}


#-------------------- list all entries -----------------------

sub list_MsgIds {
    return(keys(%MIL)) ;
}

sub list_ScriptIds {
    return(keys(%SCR)) ;
}

sub list_Actions {
    return(keys(%ACT)) ;
}

sub list_Users {
    my ( $ScriptId ) = @_ ;
	my @UserList ;

	for $UserId ( keys(%UA) ) {
		if ( is_auth($UserId,$ScriptId) ) {
		   push(@UserList,$UserId) ;
		}
	}
	return(@UserList);
}

#-------------------- get an entry -----------------------

sub get_MsgId {
    my ( $MsgId ) = @_ ;
    return ( split(':',$MIL{$MsgId}) );
}

sub get_Script {
    my ( $ScriptID) = @_ ;

    return $SCR{$ScriptID} ;
}

#----------------------- Iterators -------------------------

sub Generate_Act_Iterator {
    my ($Subject) = @_ ;
    $Subject = Case($Subject) ;

    my $Count = 0 ;
    my @Fields = split(':',$ACT{$Subject}) ; ;

    $rs = sub {
                if ( $#Fields +1 > $Count ) {
                   return ( split(';',$Fields[$Count++]) ) ;
                } else {
				   return (0) ;
				}
              } ;
    return $rs;
}

sub get_Actions {
    my ( $Subject,$Instruction ) = @_ ;
    $Subject = Case($Subject) ;

    my ($Command,$Param,$ScriptId) ;

	if ( ! defined($Actions_Iterator) ) {
	   $Actions_Iterator = Generate_Act_Iterator($Subject) ;
	} elsif ( $Instruction eq "Reset" ) {
	   $Actions_Iterator = Generate_Act_Iterator($Subject) ;
	}

    ($Command,$Param,$ScriptId) = &$Actions_Iterator();
    return($Command,$Param,$ScriptId) ;
}
#---------------------- Convenience Functions ------------
sub add_Service {
    my ( $Subject, $Command, $Param, $Script ) = @_ ;
	my $ActionId  = 1 ;
	while ( defined(&get_Script($ActionId)) ) {
	   $ActionId++ ;
	}
    $NewId = add_Action( $Subject,$Command,$Param,$ActionId ) ;
	$ActionId = $NewId if ( $NewId != 0 ) ;
	$ret = add_Script($ActionId,$Script) ;
	return($ret) ;
}

sub del_Service {
    my ( $Subject,$Command,$Param) = @_ ;
    $PgrmId = del_Action($Subject,$Command,$Param) ;
	if ( defined($PgrmId) ) {
	   &del_Pgrm($PgrmId) ;
    }
}

#---------------------- Utilities ------------------------

sub Case {
    my ( $Subject ) = @_ ;

    $Subject =~ tr/a-z/A-Z/ if ( $CaseInSubject eq "ToUpper" ) ;
	$Subject =~ tr/A-Z/a-z/ if ( $CaseInSubject eq "ToLower" ) ;

	return ($Subject) ;
}
1;
